@@ -15,9 +15,9 @@ from collections import namedtuple
 # Lazy loading
 from . import local
 
-SERIES = 'emerald-rush'
+SERIES = 'opalescent'
 
-__version__ = '5.3.4'
+__version__ = '5.4.0'
 __author__ = 'Ask Solem'
 __contact__ = 'auvipy@gmail.com'
 __homepage__ = 'https://docs.celeryq.dev/'
