@@ -14,19 +14,20 @@ This code deals with three major challenges:
 """
 import errno
 import gc
+import inspect
 import os
 import select
-import sys
 import time
 from collections import Counter, deque, namedtuple
 from io import BytesIO
 from numbers import Integral
 from pickle import HIGHEST_PROTOCOL
+from struct import pack, unpack, unpack_from
 from time import sleep
 from weakref import WeakValueDictionary, ref
 
 from billiard import pool as _pool
-from billiard.compat import buf_t, isblocking, setblocking
+from billiard.compat import isblocking, setblocking
 from billiard.pool import ACK, NACK, RUN, TERMINATE, WorkersJoined
 from billiard.queues import _SimpleQueue
 from kombu.asynchronous import ERR, WRITE
@@ -35,7 +36,7 @@ from kombu.utils.eventio import SELECT_BAD_FD
 from kombu.utils.functional import fxrange
 from vine import promise
 
-from celery.platforms import pack, unpack, unpack_from
+from celery.signals import worker_before_create_process
 from celery.utils.functional import noop
 from celery.utils.log import get_logger
 from celery.worker import state as worker_state
@@ -47,21 +48,15 @@ try:
     from _billiard import read as __read__
     readcanbuf = True
 
-    # unpack_from supports memoryview in 2.7.6 and 3.3+
-    if sys.version_info[0] == 2 and sys.version_info < (2, 7, 6):
+except ImportError:
 
-        def unpack_from(fmt, view, _unpack_from=unpack_from):  # noqa
-            return _unpack_from(fmt, view.tobytes())  # <- memoryview
-
-except ImportError:  # pragma: no cover
-
-    def __read__(fd, buf, size, read=os.read):  # noqa
+    def __read__(fd, buf, size, read=os.read):
         chunk = read(fd, size)
         n = len(chunk)
         if n != 0:
             buf.write(chunk)
         return n
-    readcanbuf = False  # noqa
+    readcanbuf = False
 
     def unpack_from(fmt, iobuf, unpack=unpack):  # noqa
         return unpack(fmt, iobuf.getvalue())  # <-- BytesIO
@@ -84,6 +79,7 @@ SCHED_STRATEGY_FAIR = 4
 
 SCHED_STRATEGIES = {
     None: SCHED_STRATEGY_FAIR,
+    'default': SCHED_STRATEGY_FAIR,
     'fast': SCHED_STRATEGY_FCFS,
     'fcfs': SCHED_STRATEGY_FCFS,
     'fair': SCHED_STRATEGY_FAIR,
@@ -95,8 +91,7 @@ Ack = namedtuple('Ack', ('id', 'fd', 'payload'))
 
 def gen_not_started(gen):
     """Return true if generator is not started."""
-    # gi_frame is None when generator stopped.
-    return gen.gi_frame and gen.gi_frame.f_lasti == -1
+    return inspect.getgeneratorstate(gen) == "GEN_CREATED"
 
 
 def _get_job_writer(job):
@@ -199,7 +194,7 @@ def iterate_file_descriptors_safely(fds_iter, source_data,
     or possibly other reasons, so safely manage our lists of FDs.
     :param fds_iter: the file descriptors to iterate and apply hub_method
     :param source_data: data source to remove FD if it renders OSError
-    :param hub_method: the method to call with with each fd and kwargs
+    :param hub_method: the method to call with each fd and kwargs
     :*args to pass through to the hub_method;
     with a special syntax string '*fd*' represents a substitution
     for the current fd object in the iteration (for some callers).
@@ -411,6 +406,9 @@ class AsynPool(_pool.Pool):
     ResultHandler = ResultHandler
     Worker = Worker
 
+    #: Set by :meth:`register_with_event_loop` after running the first time.
+    _registered_with_event_loop = False
+
     def WorkerProcess(self, worker):
         worker = super().WorkerProcess(worker)
         worker.dead = False
@@ -479,6 +477,7 @@ class AsynPool(_pool.Pool):
         )
 
     def _create_worker_process(self, i):
+        worker_before_create_process.send(sender=self)
         gc.collect()  # Issue #2927
         return super()._create_worker_process(i)
 
@@ -529,7 +528,11 @@ class AsynPool(_pool.Pool):
         for handler, interval in self.timers.items():
             hub.call_repeatedly(interval, handler)
 
-        hub.on_tick.add(self.on_poll_start)
+        # Add on_poll_start to the event loop only once to prevent duplication
+        # when the Consumer restarts due to a connection error.
+        if not self._registered_with_event_loop:
+            hub.on_tick.add(self.on_poll_start)
+            self._registered_with_event_loop = True
 
     def _create_timelimit_handlers(self, hub):
         """Create handlers used to implement time limits."""
@@ -867,7 +870,7 @@ class AsynPool(_pool.Pool):
             header = pack('>I', body_size)
             # index 1,0 is the job ID.
             job = get_job(tup[1][0])
-            job._payload = buf_t(header), buf_t(body), body_size
+            job._payload = memoryview(header), memoryview(body), body_size
             put_message(job)
         self._quick_put = send_job
 
@@ -984,10 +987,12 @@ class AsynPool(_pool.Pool):
     def flush(self):
         if self._state == TERMINATE:
             return
-        # cancel all tasks that haven't been accepted so that NACK is sent.
-        for job in self._cache.values():
-            if not job._accepted:
-                job._cancel()
+        # cancel all tasks that haven't been accepted so that NACK is sent
+        # if synack is enabled.
+        if self.synack:
+            for job in self._cache.values():
+                if not job._accepted:
+                    job._cancel()
 
         # clear the outgoing buffer as the tasks will be redelivered by
         # the broker anyway.
@@ -1003,37 +1008,45 @@ class AsynPool(_pool.Pool):
             if self._state == RUN:
                 # flush outgoing buffers
                 intervals = fxrange(0.01, 0.1, 0.01, repeatlast=True)
+
+                # TODO: Rewrite this as a dictionary comprehension once we drop support for Python 3.7
+                #       This dict comprehension requires the walrus operator which is only available in 3.8.
                 owned_by = {}
                 for job in self._cache.values():
                     writer = _get_job_writer(job)
                     if writer is not None:
                         owned_by[writer] = job
 
-                while self._active_writers:
-                    writers = list(self._active_writers)
-                    for gen in writers:
-                        if (gen.__name__ == '_write_job' and
-                                gen_not_started(gen)):
-                            # hasn't started writing the job so can
-                            # discard the task, but we must also remove
-                            # it from the Pool._cache.
-                            try:
-                                job = owned_by[gen]
-                            except KeyError:
-                                pass
+                if not self._active_writers:
+                    self._cache.clear()
+                else:
+                    while self._active_writers:
+                        writers = list(self._active_writers)
+                        for gen in writers:
+                            if (gen.__name__ == '_write_job' and
+                                    gen_not_started(gen)):
+                                # hasn't started writing the job so can
+                                # discard the task, but we must also remove
+                                # it from the Pool._cache.
+                                try:
+                                    job = owned_by[gen]
+                                except KeyError:
+                                    pass
+                                else:
+                                    # removes from Pool._cache
+                                    job.discard()
+                                self._active_writers.discard(gen)
                             else:
-                                # removes from Pool._cache
-                                job.discard()
-                            self._active_writers.discard(gen)
-                        else:
-                            try:
-                                job = owned_by[gen]
-                            except KeyError:
-                                pass
-                            else:
-                                job_proc = job._write_to
-                                if job_proc._is_alive():
-                                    self._flush_writer(job_proc, gen)
+                                try:
+                                    job = owned_by[gen]
+                                except KeyError:
+                                    pass
+                                else:
+                                    job_proc = job._write_to
+                                    if job_proc._is_alive():
+                                        self._flush_writer(job_proc, gen)
+
+                                    job.discard()
                     # workers may have exited in the meantime.
                     self.maintain_pool()
                     sleep(next(intervals))  # don't busyloop
@@ -1070,7 +1083,7 @@ class AsynPool(_pool.Pool):
                     if owner is None)
 
     def on_grow(self, n):
-        """Grow the pool by ``n`` proceses."""
+        """Grow the pool by ``n`` processes."""
         diff = max(self._processes - len(self._queues), 0)
         if diff:
             self._queues.update({
@@ -1250,7 +1263,7 @@ class AsynPool(_pool.Pool):
         """Called when a job was partially written to exited child."""
         # worker terminated by signal:
         # we cannot reuse the sockets again, because we don't know if
-        # the process wrote/read anything frmo them, and if so we cannot
+        # the process wrote/read anything from them, and if so we cannot
         # restore the message boundaries.
         if not job._accepted:
             # job was not acked, so find another worker to send it to.
