@@ -148,12 +148,6 @@ class Proxy:
     def __delitem__(self, key):
         del self._get_current_object()[key]
 
-    def __setslice__(self, i, j, seq):
-        self._get_current_object()[i:j] = seq
-
-    def __delslice__(self, i, j):
-        del self._get_current_object()[i:j]
-
     def __setattr__(self, name, value):
         setattr(self._get_current_object(), name, value)
 
@@ -199,9 +193,6 @@ class Proxy:
     def __contains__(self, i):
         return i in self._get_current_object()
 
-    def __getslice__(self, i, j):
-        return self._get_current_object()[i:j]
-
     def __add__(self, other):
         return self._get_current_object() + other
 
@@ -399,20 +390,11 @@ def getappattr(path):
     return current_app._rgetattr(path)
 
 
-def _compat_periodic_task_decorator(*args, **kwargs):
-    from celery.task import periodic_task
-    return periodic_task(*args, **kwargs)
-
-
 COMPAT_MODULES = {
     'celery': {
         'execute': {
             'send_task': 'send_task',
         },
-        'decorators': {
-            'task': 'task',
-            'periodic_task': _compat_periodic_task_decorator,
-        },
         'log': {
             'get_default_logger': 'log.get_default_logger',
             'setup_logger': 'log.setup_logger',
@@ -428,19 +410,6 @@ COMPAT_MODULES = {
             'tasks': 'tasks',
         },
     },
-    'celery.task': {
-        'control': {
-            'broadcast': 'control.broadcast',
-            'rate_limit': 'control.rate_limit',
-            'time_limit': 'control.time_limit',
-            'ping': 'control.ping',
-            'revoke': 'control.revoke',
-            'discard_all': 'control.purge',
-            'inspect': 'control.inspect',
-        },
-        'schedules': 'celery.schedules',
-        'chords': 'celery.canvas',
-    }
 }
 
 #: We exclude these from dir(celery)
@@ -528,19 +497,16 @@ def create_module(name, attrs, cls_attrs=None, pkg=None,
 
 def recreate_module(name, compat_modules=None, by_module=None, direct=None,
                     base=LazyModule, **attrs):
-    compat_modules = compat_modules or ()
+    compat_modules = compat_modules or COMPAT_MODULES.get(name, ())
     by_module = by_module or {}
     direct = direct or {}
     old_module = sys.modules[name]
     origins = get_origins(by_module)
-    compat_modules = COMPAT_MODULES.get(name, ())
 
     _all = tuple(set(reduce(
         operator.add,
         [tuple(v) for v in [compat_modules, origins, direct, attrs]],
     )))
-    if sys.version_info[0] < 3:
-        _all = [s.encode() for s in _all]
     cattrs = {
         '_compat_modules': compat_modules,
         '_all_by_module': by_module, '_direct': direct,
@@ -551,6 +517,7 @@ def recreate_module(name, compat_modules=None, by_module=None, direct=None,
     new_module.__dict__.update({
         mod: get_compat_module(new_module, mod) for mod in compat_modules
     })
+    new_module.__spec__ = old_module.__spec__
     return old_module, new_module
 
 
