@@ -1,14 +1,12 @@
 import collections
 
 import pytest
-import pytest_subtests  # noqa: F401
+import pytest_subtests  # noqa
 from kombu.utils.functional import lazy
 
-from celery.utils.functional import (DummyContext, first, firstmethod,
-                                     fun_accepts_kwargs, fun_takes_argument,
-                                     head_from_fun, lookahead, maybe_list,
-                                     mlazy, padlist, regen, seq_concat_item,
-                                     seq_concat_seq)
+from celery.utils.functional import (DummyContext, first, firstmethod, fun_accepts_kwargs, fun_takes_argument,
+                                     head_from_fun, is_numeric_value, lookahead, maybe_list, mlazy, padlist, regen,
+                                     seq_concat_item, seq_concat_seq)
 
 
 def test_DummyContext():
@@ -473,3 +471,20 @@ class test_fun_accepts_kwargs:
     ])
     def test_rejects(self, fun):
         assert not fun_accepts_kwargs(fun)
+
+
+@pytest.mark.parametrize('value,expected', [
+    (5, True),
+    (5.0, True),
+    (0, True),
+    (0.0, True),
+    (True, False),
+    ('value', False),
+    ('5', False),
+    ('5.0', False),
+    (None, False),
+])
+def test_is_numeric_value(value, expected):
+    res = is_numeric_value(value)
+    assert type(res) is type(expected)
+    assert res == expected
